package github.benjamin.equipreservebackend.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import github.benjamin.equipreservebackend.constant.DeviceReservationState;
import github.benjamin.equipreservebackend.constant.DeviceStatus;
import github.benjamin.equipreservebackend.entity.Device;
import github.benjamin.equipreservebackend.entity.Reservation;
import github.benjamin.equipreservebackend.exception.ApiException;
import github.benjamin.equipreservebackend.mapper.DeviceMapper;
import github.benjamin.equipreservebackend.response.ResponseCode;
import github.benjamin.equipreservebackend.service.DeviceService;
import github.benjamin.equipreservebackend.service.ReservationService;
import github.benjamin.equipreservebackend.utils.FileUtil;
import github.benjamin.equipreservebackend.utils.PageUtil;
import github.benjamin.equipreservebackend.vo.DeviceUserVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class DeviceServiceImpl implements DeviceService {

    private final DeviceMapper deviceMapper;

    private final ReservationService reservationService;

    @Override
    public Page<DeviceUserVO> getDeviceVO(Page<Device> pageRequest, String name) {
        LambdaQueryWrapper<Device> queryWrapper = new LambdaQueryWrapper<Device>()
                .eq(Device::getStatus, DeviceStatus.AVAILABLE)
                .orderByAsc(Device::getName);
        if (StringUtils.hasText(name)) {
            queryWrapper.like(Device::getName, name);
        }
        Page<Device> devices = deviceMapper.selectPage(pageRequest,queryWrapper);
        List<Long> deviceIds = devices.getRecords().stream()
                .map(Device::getId)
                .toList();
        List<Reservation> reservations = reservationService.getApprovedReservationsByDeviceIds(deviceIds);
        List<DeviceUserVO> deviceUserVOS = devices.getRecords().stream()
                .map(device -> buildDeviceVO(device, reservations))
                .toList();
        Page<DeviceUserVO> res = PageUtil.copyPage(devices);
        res.setRecords(deviceUserVOS);
        return res;
    }

    @Override
    public void addDevice(Device device) {
        deviceMapper.insert(device);
    }

    @Override
    public void deleteDevice(Long id) {
        deviceMapper.delete(new LambdaQueryWrapper<Device>()
                .eq(Device::getId, id));
    }

    @Override
    public Device updateDevice(Device device) {
        LambdaUpdateWrapper<Device> wrapper =new LambdaUpdateWrapper<>();
        wrapper.eq(Device::getId, device.getId());

        wrapper.set(Objects.nonNull(device.getName()), Device::getName, device.getName());
        wrapper.set(Objects.nonNull(device.getUsageRequirement()), Device::getUsageRequirement, device.getUsageRequirement());
        wrapper.set(Objects.nonNull(device.getLocation()), Device::getLocation, device.getLocation());
        wrapper.set(Objects.nonNull(device.getStatus()), Device::getStatus, device.getStatus());
        wrapper.set(Objects.nonNull(device.getTeamId()), Device::getTeamId, device.getTeamId());

        if (deviceMapper.update(wrapper) <= 0) {
            throw new ApiException(ResponseCode.UNKNOWN_ERROR, "更新设备失败");
        }

        return deviceMapper.selectOne(new LambdaQueryWrapper<Device>()
                .eq(Device::getId, device.getId()));
    }

    @Override
    public String saveImage(Long id, MultipartFile image) throws IOException {
        Device device = deviceMapper.selectOne(new LambdaQueryWrapper<Device>().eq(Device::getId, id));
        String basePath = System.getProperty("user.dir");
        if (StringUtils.hasText(device.getImagePath())) {
            Path fullPath = Paths.get(basePath, device.getImagePath()).normalize();
            File file = fullPath.toFile();
            if (file.exists() && file.isFile()) {
                file.delete();
            }
        }


        File dir = new File(basePath, "device_image");
        if (!dir.exists()) {
            dir.mkdirs();
        }

        String newFilename = FileUtil.getUniqueFileName(device.getName(), image);

        // 保存图片
        Path savePath = Paths.get(basePath, "device_image", newFilename);
        Files.copy(image.getInputStream(), savePath, StandardCopyOption.REPLACE_EXISTING);

        String imagePath = "device_image/" + newFilename;
        device.setImagePath(imagePath);
        deviceMapper.update(new LambdaUpdateWrapper<Device>()
                .eq(Device::getId, id)
                .set(Device::getImagePath, device.getImagePath()));

        return imagePath;
    }

    private DeviceUserVO buildDeviceVO(Device device, List<Reservation> reservations) {
        DeviceUserVO vo = new DeviceUserVO(device);
        // TODO 显示设备状态逻辑，根据客户需求修改
        if (reservations.stream().anyMatch(r -> Objects.equals(r.getDeviceId(), device.getId()))) {
            vo.setState(DeviceReservationState.RESERVED.getLabel());
        } else {
            vo.setState(DeviceReservationState.FREE.getLabel());
        }
        return vo;
    }
}
